//-----------------------------------------------------------------------------
// File: OptionsDlg.cpp
//
// Desc:
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
unit OptionsDlg;

interface

uses
  Windows, Messages, CommCtrl,
  Direct3D9, Resource;


//-----------------------------------------------------------------------------
// Globals variables and definitions
//-----------------------------------------------------------------------------
const
  SAMPLE_REGKEY = 'Software\Microsoft\DirectX 9.0 SDK\SHPRTPixel';
  MAX_SH_ORDER         = 6;
  MAX_LIGHTS           = 3;

  MAX_NUM_CHANNELS     = 3;


function EnumChildProc(hwnd: HWND; lParam: LPARAM): BOOL; stdcall; // CALLBACK;
function GetMsgProc(nCode: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall; // CALLBACK;
procedure GetSimulationOptionsToolTipText(nDlgId: Integer; pNMTDI: PNMTTDispInfo);
procedure GetStartupToolTipText(nDlgId: Integer; pNMTDI: PNMTTDispInfo);
procedure GetCompressToolTipText(nDlgId: Integer; pNMTDI: PNMTTDispInfo);
procedure UpdateConstText(hDlg: HWND; dwMaxVertexShaderConst: DWORD; dwTextureSize: DWORD);

const
  g_fmtTexture: array[0..1] of TD3DFormat =
  (
    D3DFMT_Q8W8V8U8,
    D3DFMT_Q16W16V16U16
  );

const
  g_dwFmt = SizeOf(g_fmtTexture) div SizeOf(g_fmtTexture[0]);

var
  g_hToolTip:     HWND = 0;
  g_hMsgProcHook: HHOOK = 0;
  g_hDlg:         HWND = 0;

implementation

uses SysUtils;

//-----------------------------------------------------------------------------
// Name: EnumChildProc
// Desc: Helper function to add tooltips to all the children (buttons/etc)
//       of the dialog box
//-----------------------------------------------------------------------------
function EnumChildProc(hwnd: HWND; lParam: LPARAM): BOOL; stdcall;
var
  ti: TToolInfo;
begin
  ti.cbSize   := SizeOf(TOOLINFO);
  ti.uFlags   := TTF_IDISHWND;
  ti.hwnd     := g_hDlg;
  ti.uId      := hwnd;
  ti.hinst    := 0;
  ti.lpszText := LPSTR_TEXTCALLBACK;
  SendMessage(g_hToolTip, TTM_ADDTOOL, 0, DWORD(@ti));

  Result:= TRUE;
end;


//-----------------------------------------------------------------------------
// Name: GetMsgProc
// Desc: msg proc hook needed to display tooltips in a dialog box
//-----------------------------------------------------------------------------
function GetMsgProc(nCode: Integer; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall; // CALLBACK;
var
  MSG, newMsg: TMsg;
begin
  MSG := PMsg(lParam)^;
  if (nCode < 0) or not IsChild(g_hDlg, MSG.hwnd) then
  begin
    Result:= CallNextHookEx(g_hMsgProcHook, nCode, wParam, lParam);
    Exit;
  end;

  case MSG.message of
    WM_MOUSEMOVE,
    WM_LBUTTONDOWN,
    WM_LBUTTONUP,
    WM_RBUTTONDOWN,
    WM_RBUTTONUP:
    begin
      if (g_hToolTip <> 0) then
      begin
        newMsg.lParam  := MSG.lParam;
        newMsg.wParam  := MSG.wParam;
        newMsg.message := MSG.message;
        newMsg.hwnd    := MSG.hwnd;
        SendMessage(g_hToolTip, TTM_RELAYEVENT, 0, Cardinal(@newMsg));
      end;
    end;
  end;

  Result:= CallNextHookEx( g_hMsgProcHook, nCode, wParam, lParam);
end;


//-----------------------------------------------------------------------------
// Name: UpdateConstText
// Desc: update the dlg's text & controls
//-----------------------------------------------------------------------------
procedure UpdateConstText(hDlg: HWND; dwMaxVertexShaderConst: DWORD; dwTextureSize: DWORD);
var
  sz: array[0..255] of Char;
  dwNumPCAVectors: DWORD;
  nIndex: Integer;
  lResult: Cardinal;
  dwNumVConsts: DWORD;
  fmtTexture: TD3DFormat;
  nBytesPerTexel: Integer;
begin
  dwNumPCAVectors := 0;
  nIndex := SendMessage(GetDlgItem(hDlg, IDC_PCA_VECTOR_COMBO), CB_GETCURSEL, 0, 0);
  lResult := SendMessage(GetDlgItem(hDlg, IDC_PCA_VECTOR_COMBO), CB_GETITEMDATA, nIndex, 0);
  if (lResult <> DWORD(CB_ERR)) then dwNumPCAVectors := lResult;

  fmtTexture := D3DFMT_UNKNOWN;
  nIndex := SendMessage(GetDlgItem(hDlg, IDC_TEXTURE_FORMAT_COMBO), CB_GETCURSEL, 0, 0);
  lResult := SendMessage(GetDlgItem(hDlg, IDC_TEXTURE_FORMAT_COMBO), CB_GETITEMDATA, nIndex, 0);
  if (HRESULT(lResult) <> CB_ERR) then fmtTexture := TD3DFormat(lResult);

  StrLFmt(sz, 256, '%d', [dwNumPCAVectors div 4]);
  sz[255] := #0;
  SetDlgItemText(hDlg, IDC_TEXTURE_NUMBER_EDIT, sz);

  nBytesPerTexel := 0;
  case fmtTexture of
    D3DFMT_Q8W8V8U8: nBytesPerTexel := 32;
    D3DFMT_Q16W16V16U16: nBytesPerTexel := 64;
  end;
  StrLFmt(sz, 256, '%d', [dwNumPCAVectors div 4 * dwTextureSize*dwTextureSize * Cardinal(nBytesPerTexel)]);
  sz[255] := #0;
  SetDlgItemText(hDlg, IDC_TEXTURE_MEMORY_EDIT, sz);

  dwNumVConsts := 1 * (1 + MAX_NUM_CHANNELS*dwNumPCAVectors div 4) + 4;
  StrLFmt(sz, 256, '%d * (1 + (3*%d/4)) + 4 = %d', [1, dwNumPCAVectors, dwNumVConsts]);
  sz[255] := #0;
  SetDlgItemText(hDlg, IDC_NUM_CONSTS_EDIT, sz);

  EnableWindow(GetDlgItem(hDlg, IDOK), (dwNumVConsts <= dwMaxVertexShaderConst));
end;


//-----------------------------------------------------------------------------
// Name: GetCompressToolTipText
// Desc: returns the tooltip text for the control
//-----------------------------------------------------------------------------
const
  ch0 = 'The number of PCA vectors controls the amount of compression done after the simulator has run. '+'There are order^2 transfer coefficients per channel. '+'So for example, with order 6 and spectral turned on '+'that would be 108 coefficients per vertex. To reduce the amount of data that needs to be passed to the vertex shader, D3DXSHPRTCompress() can be used to compress the data with a certain number of PCA vectors. The number of coefficients per '+'vertex will be reduced to the number of PCA vectors. So, for example, with 20 PCA vectors and order 6 then instead of 108 coefficients per '+'vertex we will only need 20 coefficients per vertex. The number of PCA vectors must be less than the order^2. For more detailed '+'information, see \"Clustered Principal Components for Precomputed Radiance Transfer\" by Peter-Pike Sloan, Jesse Hall, John Hart, and John Snyder to appear in '+'the Proceedings of SIGGRAPH '+'2003';
  ch2 = 'This is the maximum number of constant supported by this HW device for a vs_1_1 shader.  This is simply d3dCaps.MaxVertexShaderConst';
  ch3 = 'This is the number of vertex shader constants that will be needed by the sample''s shader given the current dialog settings.  The formula is dwNumClusters*(1+MAX_NUM_CHANNELS*dwNumPCAVectors/4)+4.';

procedure GetCompressToolTipText(nDlgId: Integer; pNMTDI: PNMTTDispInfo);
var
  str: PChar;
begin
  str := nil;
  case nDlgId of
    IDC_NUM_PCA_TEXT,
    IDC_PCA_VECTOR_COMBO:  str := ch0;

    IDC_MAX_CONSTS_EDIT,
    IDC_MAX_CONSTS_TEXT:   str := ch2;

    IDC_NUM_CONSTS_EDIT,
    IDC_NUM_CONSTS_TEXT:   str := ch3;
  end;

  pNMTDI.lpszText := str;
end;

//-----------------------------------------------------------------------------
// Name: GetSimulationOptionsToolTipText
// Desc: returns the tooltip text for the control 
//-----------------------------------------------------------------------------
procedure GetSimulationOptionsToolTipText(nDlgId: Integer; pNMTDI: PNMTTDispInfo);
const
  h1 = 'This is the file that is loaded as a ID3DXMesh* and passed into D3DXSHPRTSimulationTex() so that it can compute and return spherical harmonic transfer coefficients for each texel in a texture. '+'It returns these coefficients in a ID3DXBuffer*. This process takes '+'some time and should be precomputed, however the results can be used in real time. For more detailed information, see \"Precomputed Radiance Transfer for Real-Time Rendering in Dynamic, Low-Frequency Lighting '+'Environments\" by Peter-Pike Sloan, Jan Kautz, and John Snyder, SIGGRAPH 2002.';
  h2 = 'This controls the number of spherical harmonic basis functions used. The simulator generates order^2 coefficients per channel. Higher order allows for higher frequency lighting environments '+'which allow for sharper shadows with the tradeoff of more coefficients per vertex that need to be processed by the vertex shader. For convex objects (no shadows), 3rd order has very '+'little approximation error.  For more detailed information, see \"Spherical Harmonic Lighting: The '+'Gritty Details\" by Robin Green, GDC 2003 and \"An Efficient Representation of Irradiance Environment Maps\" by Ravi Ramamoorthi, and Pat Hanrahan, SIGGRAPH 2001.';
  h3 = 'This controls the number of bounces simulated. If this is non-zero then inter-reflections are calculated. Inter-reflections are, for example, when a light shines on a red wall and bounces on '+'a white wall. The white wall even though it contains no red in the material will reflect some red do to the bouncing of the light off the red wall.';
  h4 = 'This controls the number of rays to shoot at each sample. The more rays the more accurate the final result will be, but it will increase time it takes to precompute the transfer coefficients.';
  h5 = 'If checked then subsurface scattering will be done in the simulator. Subsurface scattering is when light penetrates a translucent surface and comes out the other side. For example, a jade sculpture or a flashlight '+'shining through skin exhibits subsurface scattering. The simulator assumes the mesh is made of a homogenous material. If subsurface scattering is not used, then the length scale, the relative '+'index of refraction, the reduced scattering coefficients, and the absorption coefficients are not used.';
  h6 = 'If checked then the simulator will process 3 channels: red, green, and blue and return order^2 spherical harmonic transfer coefficients for each of these channels in a single ID3DXBuffer* buffer. Otherwise it use '+'values of only one channel (the red channel) and return the transfer coefficients for just that single channel. A single channel is useful for lighting environments that don''t need to have the whole '+'spectrum of light such as shadows';
  h7 = 'These are some example materials. Choosing one of these materials with change the all the material values below. The parameters for these materials are from \"A Practical Model for Subsurface Light Transport\" by Henrik '+'Wann Jensen, Steve R. Marschner, Marc Levoy, Pat Hanrahan, SIGGRAPH 2001. The relative index of refraction is with respect the material immersed in air.';
  h8 = 'The reduced scattering coefficient is a parameter to the volume rendering equation used to model light propagation in a participating medium. For more detail, see \"A Practical Model for Subsurface Light Transport\" by '+'Henrik Wann Jensen, Steve R. Marschner, Marc Levoy, Pat Hanrahan, SIGGRAPH 2001';
  h9 = 'The absorption coefficient is a parameter to the volume rendering equation used to model light propagation in a participating medium. For more detail, see \"A Practical Model for Subsurface Light Transport\" by Henrik Wann'+' Jensen, Steve R. Marschner, Marc Levoy, Pat Hanrahan, SIGGRAPH 2001';
  h10= 'This will start the simulator based on the options selected above. This process takes some time and should be precomputed, however the results can be used in real time. When the simulator is done calculating the spherical harmonic transfer '+'coefficients for each texel, the sample will use D3DXSHPRTCompress() to compress the data based on the number of PCA vectors chosen. This sample will then save the binary data of coefficients to a file. This sample will also allow you '+'to view the results by passing these coefficients as a texture to a pixel shader for real time rendering (if the Direct3D device has '+'hardware accelerated programmable pixel shader support). This sample also stores the settings of this dialog to the registry so it remembers them for next time.';
var
  str: PChar;
begin
  str := nil;
  case nDlgId of
    IDC_INPUT_MESH_TEXT,
    IDC_MESH_NAME: str := h1;

    IDC_ORDER_TEXT,
    IDC_ORDER_SLIDER: str := h2;

    IDC_TEXTURE_SIZE_TEXT,
    IDC_TEXTURE_SIZE_COMBO: str := 'This is the size of the width and height of the texture.  The larger this number the more accurate the final result will be, but it will increase time it takes to precompute the transfer coefficients and require the more texture memory.';

    IDC_NUM_BOUNCES_TEXT,
    IDC_NUM_BOUNCES_EDIT,
    IDC_NUM_BOUNCES_SPIN: str := h3; // TEXT("This controls the number of bounces simulated. If this is non-zero then inter-reflections are calculated. Inter-reflections are, for example, when a light shines on a red wall and bounces on a white wall. The white wall even though it contains no red in the material will reflect some red do to the bouncing of the light off the red wall."); break;

    IDC_NUM_RAYS_TEXT,
    IDC_NUM_RAYS_EDIT,
    IDC_NUM_RAYS_SPIN: str := h4; //TEXT("This controls the number of rays to shoot at each sample. The more rays the more accurate the final result will be, but it will increase time it takes to precompute the transfer coefficients."); break;

    IDC_SUBSURF_CHECK: str := h5; // TEXT("If checked then subsurface scattering will be done in the simulator. Subsurface scattering is when light penetrates a translucent surface and comes out the other side. For example, a jade sculpture or a flashlight shining through skin exhibits subsurface scattering. The simulator assumes the mesh is made of a homogenous material. If subsurface scattering is not used, then the length scale, the relative index of refraction, the reduced scattering coefficients, and the absorption coefficients are not used."); break;

    IDC_SPECTRAL_CHECK: str := h6; // TEXT("If checked then the simulator will process 3 channels: red, green, and blue and return order^2 spherical harmonic transfer coefficients for each of these channels in a single ID3DXBuffer* buffer. Otherwise it use values of only one channel (the red channel) and return the transfer coefficients for just that single channel. A single channel is useful for lighting environments that don't need to have the whole spectrum of light such as shadows"); break;

    IDC_RED_TEXT: str := 'The values below are the red coefficients. If spectral is turned off, then this is the channel that will be used.';
    IDC_GREEN_TEXT: str := 'The values below are the green coefficients';
    IDC_BLUE_TEXT: str := 'The values below are the blue coefficients';

    IDC_PREDEF_COMBO,
    IDC_PREDEF_TEXT: str := h7; // TEXT("These are some example materials. Choosing one of these materials with change the all the material values below. The parameters for these materials are from \"A Practical Model for Subsurface Light Transport\" by Henrik Wann Jensen, Steve R. Marschner, Marc Levoy, Pat Hanrahan, SIGGRAPH 2001. The relative index of refraction is with respect the material immersed in air."); break;

    IDC_REFRACTION_TEXT,
    IDC_REFRACTION_EDIT: str := 'Relative index of refraction is the ratio between two absolute indexes of refraction. An index of refraction is ratio of the sine of the angle of incidence to the sine of the angle of refraction.';

    IDC_LENGTH_SCALE_TEXT,
    IDC_LENGTH_SCALE_EDIT: str := 'When subsurface scattering is used the object is mapped to a cube of length scale mm per side. For example, if length scale is 10, then the object is mapped to a 10mm x 10mm x 10mm cube.  The smaller the cube the more light penetrates the object.';

    IDC_SCATTERING_TEXT,
    IDC_SCATTERING_G_EDIT,
    IDC_SCATTERING_B_EDIT,
    IDC_SCATTERING_R_EDIT: str := h8; // TEXT("The reduced scattering coefficient is a parameter to the volume rendering equation used to model light propagation in a participating medium. For more detail, see \"A Practical Model for Subsurface Light Transport\" by Henrik Wann Jensen, Steve R. Marschner, Marc Levoy, Pat Hanrahan, SIGGRAPH 2001"); break;

    IDC_ABSORPTION_TEXT,
    IDC_ABSORPTION_G_EDIT,
    IDC_ABSORPTION_B_EDIT,
    IDC_ABSORPTION_R_EDIT: str := h9; // TEXT("The absorption coefficient is a parameter to the volume rendering equation used to model light propagation in a participating medium. For more detail, see \"A Practical Model for Subsurface Light Transport\" by Henrik Wann Jensen, Steve R. Marschner, Marc Levoy, Pat Hanrahan, SIGGRAPH 2001"); break;

    IDC_REFLECTANCE_TEXT,
    IDC_REFLECTANCE_R_EDIT,
    IDC_REFLECTANCE_B_EDIT,
    IDC_REFLECTANCE_G_EDIT: str := 'The diffuse reflectance coefficient is the fraction of diffuse light reflected back. This value is typically between 0 and 1.';

    IDC_OUTPUT_TEXT,
    IDC_OUTPUT_EDIT: str := 'This sample will save a binary buffer of spherical harmonic transfer coefficients to a file which the sample can read in later.  This is the file name of the where the resulting binary buffer will be saved';

    IDC_OUTPUT_BROWSE_BUTTON: str := 'Select the output file name';

    IDOK: str := h10; // TEXT("This will start the simulator based on the options selected above. This process takes some time and should be precomputed, however the results can be used in real time. When the simulator is done calculating the spherical harmonic transfer coefficients for each vertex, the sample will use D3DXSHPRTCompress() to compress the data based on the number of PCA vectors chosen. This sample will then save the binary data of coefficients to a file. This sample will also allow you to view the results by passing these coefficients to a vertex shader for real time rendering (if the Direct3D device has hardware accelerated programmable vertex shader support). This sample also stores the settings of this dialog to the registry so it remembers them for next time."); break;

    IDCANCEL: str := 'This cancels the dialog, does not save the settings, and does not run the simulator.';
  end;

  pNMTDI.lpszText := str;
end;


//-----------------------------------------------------------------------------
// Name: GetStartupToolTipText
// Desc: returns the tooltip text for the control
//-----------------------------------------------------------------------------
procedure GetStartupToolTipText(nDlgId: Integer; pNMTDI: PNMTTDispInfo);
var
  str: PChar;
const
  h1 = 'Precomputed radiance transfer (PRT) using low-order spherical harmonic (SH) basis functions have a number of advantages over typical diffuse (N dot L) lighting.  Area light sources, interreflections, soft shadows, self shadowing, and subsurface '+'scattering can be rendered in real time after a precomputed simulation.';
  h2 = 'The approach is to precompute for a given object an expensive light transport simulation that simulates shadows, interreflections, and subsurface scattering.  For each vertex or texel, the results are recorded as transfer coefficients for the SH '+'basis functions.  Then in real time the incident radiance is computed as another set of coefficients of the SH basis functions.  These two sets of coefficients are used in real time by the shader to create the lighting for the object.';
begin
  str := nil;
  case nDlgId of
    IDC_WHY: str := h1; // TEXT("Precomputed radiance transfer (PRT) using low-order spherical harmonic (SH) basis functions have a number of advantages over typical diffuse (N dot L) lighting.  Area light sources, interreflections, soft shadows, self shadowing, and subsurface scattering can be rendered in real time after a precomputed simulation."); break;

    IDC_HOW: str := h2; // TEXT("The approach is to precompute for a given object an expensive light transport simulation that simulates shadows, interreflections, and subsurface scattering.  For each vertex or texel, the results are recorded as transfer coefficients for the SH basis functions.  Then in real time the incident radiance is computed as another set of coefficients of the SH basis functions.  These two sets of coefficients are used in real time by the shader to create the lighting for the object."); break;

    IDC_WHAT: str := 'This sample demonstrates how use D3DXSHPRTSimulationTex(), a texel based SH PRT simulator and records the results to a file.  It can then view the results in result time with a ps_2_0 pixel shader.';

    IDC_RUN_PRT_SIM: str := 'Run the PRT SH simulator after setting up its parameters';

    IDC_INPUT_MESH_TEXT,
    IDC_INPUT_MESH_EDIT: str := 'The file name of the mesh to render.';

    IDC_INPUT_MESH_BROWSE_BUTTON: str := 'Select the input mesh file name';

    IDC_SIM_RESULTS_TEXT,
    IDC_SIM_RESULTS_EDIT: str := 'Previously saved results of the simulator for this mesh.  The simulator results are used in real time in the pixel shader to light the mesh.  If the results from a different mesh are used then either an error will occur or the lighting will be incorrect.';

    IDC_SIM_RESULTS_BROWSE_BUTTON: str := 'Select the simulation results file name';

    IDC_VIEW_RESULTS: str := 'View the mesh with a ps_2_0 pixel shader using previously saved simulator results.';

    IDCANCEL: str := 'Cancel the dialog';
  end;

  pNMTDI.lpszText := str;
end;

end.

